import argparse
import logging
import os
import sys

import manik.semantic.api as semantic
import manik.lexical.api as lexical
import manik.scan as scan

# DEBUG CODE REMOVE
import manik.semantic.ma_preprocessor_2 as prepro


logger = logging.getLogger("manik")


def main():

    # create the command line interface
    argparser = argparse.ArgumentParser(
        prog="manik",
        description="Scans .MA files for potential virus attacks",
        epilog="(c) 2020-22 Tech Art Foundation & Team Manik.\n"
        "Contribute at https://github.com/techartorg/manik",
    )

    argparser.add_argument(
        "--file", "-f", nargs="+", help="file or folder to scan", required=True
    )
    argparser.add_argument("--log", "-l", nargs="?", help="Path to optional log file")
    argparser.add_argument(
        "--scan",
        "-s",
        choices=["semantic", "lexical"],
        default="semantic",
        help="specify the scanning strategy",
    )
    argparser.add_argument(
        "--verbose", "-v", action="store_true", help="display processing"
    )

    argparser.add_argument(
        "--debug", "-d", action="store_true", help="print the results of ths script preprocessor"
    )


    options = argparser.parse_args()

    # if we add more scanner types we'll need to plug them in here or make
    # a registration mechanism
    _scanner = None

    if options.debug:
        _scanner = scan.MAScanner.from_module(prepro)
    else:
        if options.scan == "semantic":
            _scanner = scan.MAScanner.from_module(semantic)
        else:
            _scanner = scan.MAScanner.from_module(lexical)

    results = []

    if options.log:
        logger.addHandler(logging.FileHandler(options.log))

    if options.verbose:
        logger.setLevel(logging.DEBUG)




    for f in options.file:
        if not os.path.exists(f):
            logger.warning("file '{}' does not exist".format(f))
        elif os.path.isdir(f):
            results.append(_scanner.scan_folders(f))
        else:
            results.append({f: _scanner.scan_file(f)})

    if results:
        sys.exit(1)


if __name__ == "__main__":
    main()
