from collections import namedtuple
import logging

from manik.lexical import command_scanner, ma_preprocessor


logger = logging.getLogger("manik.lexical.api")

_scan_funcs = (
    command_scanner.scan_statement_plaintext,
    command_scanner.scan_statement_modified,
    command_scanner.scan_statement_commands,
)


def get_parser():
    return ma_preprocessor.parse_stream


def get_scanner():
    return scan


MelStatementBase = namedtuple("mel", "statement lineno")


class MelStatement(MelStatementBase):
    def __repr__(self):
        truncated = self.statement[:80] + ('..' if len(self.statement) > 80 else '')
        return "line {}: {}".format(self.lineno, truncated)


def normalize(result):
    return result[0], MelStatement(result[1], -1)


def scan(chunk):
    for func in _scan_funcs:
        result = func(chunk)
        if result:
            return normalize(result)

    return None
